
PLANT_STATE="plant"

local GRASS_PLANT = {["grass"] = true, ["oat"] = true, ["wheat"] = true, ["corn"] = true,}

local function getPastureCellPos3D(pos2d)
	return Vec3(pos2d.x, getTerrainHeight(pos2d.x, pos2d.y), pos2d.y)
end

local function playerGrow(w, b, state, drain, growValue, index, checkAction)	
	local plantBuild = b
	local cellNow = plantBuild:GetPlantCell(index)
	local cellsCount = plantBuild.plantCellCount 
		
	
	if cellNow.state == state then	
		local humidity = plantBuild.humidity
		if humidity > plantBuild.plantCfg.humidityMax then
			if checkAction == false then
				w:addTaskSetHint("@hint_plant_drain@")
			
				w:addTaskAttachModel("tools-shovel-01")
				w:addTaskPlayAnimationOnce("hilling")
				w:addTaskAttachModel("")
				w:addTaskPlayAnimation("stay")
				w:addTaskCallEvent(b, "Drain", drain, 0); 
			
				w:addTaskResetHint()
			end
			return true;
		end
	
		if checkAction == false then
			w:addTaskSetHint("@hint_plant_grow@")
			
			w:addTaskReservePlantCell(b, index, true)
				
			local cellPos = getPastureCellPos3D(cellNow.pos)
			if cellPos.x ~= w.pos.x or cellPos.z ~= w.pos.y then
				local dir = Vec2Normalize(Vec2(cellPos.x, cellPos.z) - w.pos)
				w:addTaskRotate(dir)
			end
	
			local cell = plantBuild:GetPlantCell(index)
			w:addTaskAttachModel("tools-shovel-01")
			w:addTaskPlayAnimationOnce("hilling")
			--w:addTaskWork(3000, "hilling")
			w:addTaskAttachModel("")
			w:addTaskPlayAnimation("stay")
			w:addTaskCallEvent(b, "Grow", index, growValue);
			w:addTaskReservePlantCell(b, index, false)
			
			w:addTaskResetHint()
		end
		return true
	end	
		
	return false;
end
	
local function grow(w, b, state, drain, water, growValue)
	local index = -1;
	local minValue = 999999999;
			
	local plantBuild = b
	local cellsCount = plantBuild.plantCellCount 
	
	local temperatureBad = false
	if game.temperature < plantBuild.plantCfg.temperatureMin then
		temperatureBad = true
	end
	
	for i = 0, cellsCount-1, 1 do
		local cell = plantBuild:GetPlantCell(i)
		
		if cell.state == state then		
			if temperatureBad then
				return true
			end
					
			if cell.reserve == false then
				if minValue > cell.time then
					index = i
					minValue = cell.time
				end
			end	
		end
	end
	
	if index >= 0 then
		local humidity = plantBuild.humidity
		if humidity > plantBuild.plantCfg.humidityMax then	
			
			local partCount = 3
			local drainPart = drain/partCount
			
			for i = 0, partCount-1, 1 do
				local cell = plantBuild:GetPlantCell(math.random(cellsCount)-1)
				w:addTaskSetHint("@hint_plant_drain@")
			
				goTo(w, getPastureCellPos3D(cell.pos))
				w:addTaskAttachModel("tools-shovel-01")
				w:addTaskPlayAnimationOnce("hilling")
				w:addTaskAttachModel("")
				w:addTaskPlayAnimation("stay")
				w:addTaskCallEvent(b, "Drain", drainPart, 0); 
			
				w:addTaskResetHint()
			end
			return true;
		end
			
		if humidity < plantBuild.plantCfg.humidityMin then
			local barn = findBarnWithRes(w, b.pos, "water", config.findResourcesRadius)
			if barn ~= nil then
				local transformCount = math.min(barn:getResourceFree("water"), getMaxTransportCount(w, "water"))		
						
				if transformCount > 0 then
					w:addTaskSetHint("@hint_plant_water@")

					getResourceOptimized(w, barn, "water", transformCount)
					
					local partCount = 3
					local waterPart = water/partCount
				
					for i = 0, partCount-1, 1 do
						local cell = plantBuild:GetPlantCell(math.random(cellsCount)-1)
						goToWithAnimation(w, getPastureCellPos3D(cell.pos), getResourceTransportAnimation("water"))
						w:addTaskWork(300, "stay")
						w:addTaskCallEvent(b, "Water", waterPart, 0) 
					end
				
					w:addTaskResetHint()
				
					return true;
				end
			end
		end
			
		w:addTaskReservePlantCell(b, index, true)
		local cell = plantBuild:GetPlantCell(index)
		
		w:addTaskSetHint("@hint_plant_grow@")
			
		goTo(w, getPastureCellPos3D(cell.pos))
		w:addTaskAttachModel("tools-shovel-01")
		w:addTaskPlayAnimationOnce("hilling")
		--w:addTaskWork(3000, "hilling")
		w:addTaskAttachModel("")
		w:addTaskPlayAnimation("stay")
		w:addTaskCallEvent(b, "Grow", index, growValue);
		w:addTaskReservePlantCell(b, index, false)
		
		w:addTaskResetHint()
				
		return true;
	end
		
	return false;
end

local function stateChangeFunc(b)
	if b.state ~= WORKING then
		b:hideIcon(BUILDING_NO_WARM)
	end
end

local generateFence = function(b)
	local wHalf = math.floor(b.widthDefault/2)
	local lHalf = math.floor(b.lengthDefault/2)
	local prefabs = { "models/house/fence1_2.prefab", "models/house/fence1_3.prefab", "models/house/fence1_4.prefab" }

	addFence(b, Vec2(0+0.5, 0+0.5), Vec2(wHalf-0.5, 0+0.5), prefabs)	
	addFence(b, Vec2(wHalf+1.5, 0+0.5), Vec2(b.widthDefault-0.5, 0+0.5), prefabs)	
	
	addFence(b, Vec2(0+0.5,  b.lengthDefault-0.5), Vec2(wHalf-0.5,  b.lengthDefault-0.5), prefabs)
	addFence(b, Vec2(wHalf+1.5,  b.lengthDefault-0.5), Vec2(b.widthDefault-0.5,  b.lengthDefault-0.5), prefabs)
	
	addFence(b, Vec2(0+0.5, 0+0.5), Vec2(0+0.5, lHalf-0.5), prefabs)
	addFence(b, Vec2(0+0.5, lHalf+1.5), Vec2(0+0.5, b.lengthDefault-0.5), prefabs)
	
	addFence(b, Vec2(b.widthDefault-0.5, 0+0.5), Vec2(b.widthDefault-0.5, lHalf-0.5), prefabs)
	addFence(b, Vec2(b.widthDefault-0.5, lHalf+1.5), Vec2(b.widthDefault-0.5, b.lengthDefault-0.5), prefabs)
end

local function prefabChangeFunc(b)
	updateFence(b, generateFence)
end

Plant = Class("Plant", BuildingPlanting)
  
Plant.updateBuild = buildFunction
   
Plant.createWindow = function(b)
	return createWindow(BLD_WIN_FIELD, "Field.layout")
end
       
Plant.onStateChange = stateChangeFunc
Plant.onPrefabChange = prefabChangeFunc
	      
Plant.init = function(b)
	b:setWorkMode(PLANT_STATE, "seed")
	local data = b:data()
	data["timeIcon"]=0
	data["tempTimeIcon"]=0
	initFence(b)
end
  
Plant.onEvent = function(b, s, p1, p2)
	local plantBuild = b
	local cell = plantBuild:GetPlantCell(p1)
	if s == "Seed" then
		cell.state = GROWTH
	elseif s == "Grow" then
		if plantBuild.plantCfg then
			if game.temperature >= plantBuild.plantCfg.temperatureMin then
				cell:addTime(p2, true)	
			end
		end
	elseif s == "Chop" then
		cell.state = SEED
	elseif s == "Drain" then
		if plantBuild.plantCfg then
			if plantBuild.humidity > plantBuild.plantCfg.humidityMin then	
				local waterMax = plantBuild.waterMax
				local correctWater = (plantBuild.plantCfg.humidityMax*waterMax + plantBuild.plantCfg.humidityMin*waterMax)/2 - plantBuild.water
				plantBuild.water = plantBuild.water + math.max(correctWater, p1)
			end
		end
	elseif s == "Water" then
		if plantBuild.plantCfg then
			if plantBuild.humidity < plantBuild.plantCfg.humidityMax then
				local waterMax = plantBuild.waterMax
				local correctWater = (plantBuild.plantCfg.humidityMax - plantBuild.humidity)*waterMax
				plantBuild.water = plantBuild.water + math.min(correctWater, p1)
			end
		end
	elseif s == "Harvest" then
		local state = SEED
		if cell.harvestCount > 1 then
			cell.harvestCount = cell.harvestCount - 1
			state = RIPEN;
		else
			cell.harvestCount = 0;
			if plantBuild.plantCfg.harvestCount ~= 1 then
				state = CHOP;
			end
		end

		cell.state = state;
	end
end
  
Plant.updateWorker = function(b, w)
	
	if repair(w, b) then
		return true
	end
	
	local transfer = false
	local workMode = b:getWorkMode(PLANT_STATE)
	if workMode == "seed" or workMode == "ripen" then
		transfer = true
	end
	
	local plantBuild = b
	if plantBuild.plantCfg == nil then
		return true;	
	end
	
	local goodTemperature = true
	local liveTemperature = true
	local temperature = game.temperature
	if temperature <= config.plantDieTemp then
		liveTemperature = false	
		transfer = true		
	end
	if temperature < plantBuild.plantCfg.temperatureMin then
		goodTemperature = false	
	end
	
	local res = getBuildingResources(b)			
	for key,value in pairs(res) do
		local count = math.max(value + b:getPickReservation(key), 0)
		local maxGetCount = getMaxTransportCount(w, key)
		
		if count >= maxGetCount or transfer then
			if count > 0 then
				local transformCount = math.min(count, maxGetCount)
				local barn = findBarnToDeposit(w, b.pos, key, transformCount)
				if barn ~= nil then
					w:addTaskSetHint("@hint_store_res@")

					local cellNow = plantBuild:getCellByPos(w.pos)
					if cellNow == -1 then
						--goTo(w, b.buildingCenter+Vec3(b.width/2,0,b.length/2))
						local bldPos = b.pos
						goTo(w, Vec3(bldPos.x, getTerrainHeight(bldPos.x, bldPos.y), bldPos.y))
					end
					takeResource(w, b.uid, key, transformCount)
					putResource(w, barn, key, transformCount)
					w:addTaskPlayAnimation("stay")	
					
					w:addTaskResetHint()

					b:hideIcon(BUILDING_NO_BARN)					
					return true;
				else
					b:showIcon(BUILDING_NO_BARN)
				end	
			end
		end

	end
	
	if liveTemperature == false and plantBuild.plantCfg.harvestCount > 1 then
		return
	end
	
	local cellsCount = plantBuild.plantCellCount
	
	local widthCells = plantBuild.widthCells
	local lengthCells = plantBuild.lengthCells
	local widthPlant = plantBuild.horizontalBeds
	
	local nextStatus = true;
	while nextStatus do
		local workStatus = b:getWorkMode(PLANT_STATE)
		
		if workStatus == "seed" then
			
			local freeCellsCount = 0
			local freeCells = {}
			for i = 0, cellsCount-1, 1 do
				local cell = plantBuild:GetPlantCell(i)
				
				if cell.state == SEED then				
					nextStatus = false;
					
					if cell.reserve == false then
						freeCells[freeCellsCount] = i
						freeCellsCount = freeCellsCount + 1
					end

				end
			end
			
			if goodTemperature == false then -- or game.season == SEASON_FALL
				if freeCellsCount == cellsCount then
					nextStatus = false;
				else
					nextStatus = true;
				end
			else	
				local cellNow = plantBuild:getCellByPos(w.pos)
				--if cellNow == -1 then
				--	goToBuildingOptimized(w, b)
				--	w:addTaskPlayAnimation("stay")
				--end
				
				local nextCell = -1
				if cellNow ~= -1 then
					if widthPlant then
						if math.fmod(cellNow, widthCells) < widthCells-1 then
							local cell = plantBuild:GetPlantCell(cellNow + 1)
							if cell.state == SEED then	
								if cell.reserve == false then
									nextCell = cellNow + 1
								end
							end
						end
					else
						if cellNow + widthCells < cellsCount then
							local cell = plantBuild:GetPlantCell(cellNow + widthCells)
							if cell.state == SEED then	
								if cell.reserve == false then
									nextCell = cellNow + widthCells
								end
							end
						end
					end
				end

				if nextCell == -1 then
					if widthPlant then
						local rowCount = cellsCount/widthCells
						for i = 0, rowCount-1, 1 do
							local firstRowCell = i*widthCells
							local cell = plantBuild:GetPlantCell(firstRowCell)
							if cell.state == SEED then	
								if cell.reserve == false then
									nextCell = firstRowCell
									break
								end
							end
						end	
					else
						local rowCount = widthCells
						for i = 0, widthCells-1, 1 do
							local firstRowCell = i
							local cell = plantBuild:GetPlantCell(firstRowCell)
							if cell.state == SEED then	
								if cell.reserve == false then
									nextCell = firstRowCell
									break
								end
							end
						end	
					end		
				end
			
				if nextCell == -1 then
					if freeCellsCount > 0 then
						nextCell = freeCells[math.random(freeCellsCount)-1]
					end
				end
			
				if nextCell ~= -1 then
					local cell = plantBuild:GetPlantCell(nextCell)
					
					w:addTaskSetHint("@hint_plant_seed@")
			
					w:addTaskReservePlantCell(b, nextCell, true)
					goToPivot(w, getPastureCellPos3D(cell.pos)) --goTo(w, getPastureCellPos3D(cell.pos))
					w:addTaskAttachModel("tools-shovel-01")
					w:addTaskPlayAnimationOnce("plant")
					w:addTaskPlayAnimation("stay")
					w:addTaskAttachModel("")
					w:addTaskCallEvent(b, "Seed", nextCell, 0);
					w:addTaskReservePlantCell(b, nextCell, false)
					
					w:addTaskResetHint()
						
					return true;
				end	
			end			
		end
		
		if workStatus == "growth" then
			if grow(w, b, GROWTH, -20000, 20000, 150000) then
				return true;
			end
		end
		
		if workStatus == "ripen" then
			if grow(w, b, RIPEN, -20000, 20000, 110000) then
				return true;
			end
		end
		
		if workStatus == "harvest" then	

			local index = -1;
			local maxValue = 0;
			
			for i = 0, cellsCount-1, 1 do
				local cell = plantBuild:GetPlantCell(i)
				
				if cell.harvestCount == plantBuild.harvestCount then
					local plantCfg = cell.plantCfg
					if plantCfg ~= nil then
						if cell.state == HARVEST then
							nextStatus = false;
					
							if resourceLimitReached(plantCfg.outResource) then
								return true;
							end
			
							if cell.reserve == false then
					
								if b.freeSpace < cell.resourceCount then
									return true
								end
						
								if cell.resourceCount >= maxValue then
									maxValue = cell.resourceCount
									index = i
								end
							end
						end
					end					
				end
			end
			
			if index >= 0 then			
				local cell = plantBuild:GetPlantCell(index)			
				
				w:addTaskSetHint("@hint_plant_harvest@")
			
				w:addTaskReservePlantCell(b, index, true)
				goTo(w, getPastureCellPos3D(cell.pos))
				if cell.plantCfg.field then
					if GRASS_PLANT[cell.plantCfg.name] then
					w:addTaskAttachModel("tools-scythe-01")
					w:addTaskPlayAnimationOnce("axecut")
					else
					w:addTaskAttachModel("tools-shovel-01")
					w:addTaskPlayAnimationOnce("plant")
					end
				else
					w:addTaskAttachModel("")
					w:addTaskWork(2000, "work")
				end					
				w:addTaskAttachModel("")
				w:addTaskCallEvent(b, "Harvest", index, 0)
				w:addTaskPlayAnimation("stay")
				w:addTaskAddResourceToBuilding(b, cell.plantCfg.outResource, cell.resourceCount)
				w:addTaskReservePlantCell(b, index, false)	
				
				w:addTaskResetHint()
			
				return
			end
			
		end
		
		if workStatus == "chop" then
			for i = 0, cellsCount-1, 1 do
				local cell = plantBuild:GetPlantCell(i)
				
				if cell.state == CHOP then
					nextStatus = false;
					
					if cell.reserve == false then
								
						w:addTaskSetHint("@hint_plant_chop@")
				
						w:addTaskReservePlantCell(b, i, true)
						goTo(w, getPastureCellPos3D(cell.pos))
						
						if cell.plantCfg.field then
							if GRASS_PLANT[cell.plantCfg.name] then
							w:addTaskAttachModel("tools-scythe-01")
							w:addTaskPlayAnimationOnce("axecut")
							else
							w:addTaskAttachModel("tools-shovel-01")
							w:addTaskPlayAnimationOnce("hilling")
							end
						else
							w:addTaskAttachModel("tools-axe-01")
							w:addTaskWork(2000, "axecut")
						end
						
						w:addTaskCallEvent(b, "Chop", i, 0);
						w:addTaskReservePlantCell(b, i, false)	
						w:addTaskPlayAnimation("stay")	
						w:addTaskAttachModel("")						
						
						w:addTaskResetHint()
						return true;
					end
					
				end
			end
		end
		
		if nextStatus then	
			if workStatus == "seed" then
				b:setWorkMode(PLANT_STATE, "growth")
			elseif workStatus == "growth" then
				plantBuild.harvestCount = plantBuild.plantCfg.harvestCount
				b:setWorkMode(PLANT_STATE, "ripen")
			elseif workStatus == "ripen" then
				b:setWorkMode(PLANT_STATE, "harvest")
			elseif workStatus == "harvest" then
				plantBuild.harvestCount = plantBuild.harvestCount-1;
				if plantBuild.harvestCount > 0 then
					b:setWorkMode(PLANT_STATE, "ripen")
				else
					b:setWorkMode(PLANT_STATE, "chop")
				end
			elseif workStatus == "chop" then
				b:setWorkMode(PLANT_STATE, "seed")
				return true;
			end
		end
	end	
end

Plant.update = function(b, dt)
	updateLimitIcon(b, b:getWorkMode(BLD_MODE_WORK_RES), dt)
end

Plant.updateWork = function(b, dt)

	local data = b:data()
	local elapsedTime = data["tempTimeIcon"] + dt
	
	if elapsedTime > 1000 then
		elapsedTime = 0
				
		if b.state == WORKING then
			local iconTempShow = false	

			local plantBuild = b
			if plantBuild.plantCfg then 
				local tempMin = plantBuild.plantCfg.temperatureMin
				if game.season == SEASON_FALL then -- если осень
					tempMin = tempMin + 8
				end
				
				if game.temperature < tempMin then				
					local cellsCount = plantBuild.plantCellCount
					for i = 0, cellsCount-1, 1 do
						local cell = plantBuild:GetPlantCell(i)				
						local plantCfg = cell.plantCfg
						if plantCfg ~= nil then
							local state = cell.state
							if state == HARVEST then -- если сбор урожая
								iconTempShow = true
								break
							elseif state == GROWTH then -- если растет не многолетнее растение
								if plantCfg.harvestCount == 1 then
									iconTempShow = true
									break
								end
							elseif state == RIPEN then -- если созревает
								iconTempShow = true
								break
							end
						end
					end
					
					if iconTempShow == false then
						if b.workerCount > 0 then -- если рабочие есть
							if plantBuild.plantCfg.harvestCount == 1 then
								iconTempShow = true
							end
						end
					end
				end
			else
				if b.workerCount > 0 then -- если рабочие есть
					if game.temperature <= config.plantDieTemp then
						iconTempShow = true
					end
				end
			end				
		
			if iconTempShow then
				b:showIcon(BUILDING_NO_WARM)
			else
				b:hideIcon(BUILDING_NO_WARM)
			end
		else
			b:hideIcon(BUILDING_NO_WARM)
		end
	end
		
	data["tempTimeIcon"] = elapsedTime
end

Plant.updatePlayer = function(b, w, checkAction)
	if w.storageEmpty == false then
		local resType = w.resourceInStorage
		if resType ~= "water" then
			return
		end
		
		if checkAction == false then
			
			w:addTaskSetHint("@hint_plant_water@")
			
			w:addTaskWork(300, "stay")
			w:addTaskCallEvent(b, "Water", 20000, 0)
			local data = {}
			w:addTaskScript("taskClearInventory", data)
			
			w:addTaskResetHint()
		end
		return true
	end
	
	local plantBuild = b
	if plantBuild.plantCfg == nil then
		return false
	end
	
	local cellIndexNow = plantBuild:getCellByPos(w.pos)
	if cellIndexNow == -1 then
		return false
	end
	
	local cellNow = plantBuild:GetPlantCell(cellIndexNow)
	if cellNow.reserve then
		return false
	end
				
	local transfer = false
	local workMode = b:getWorkMode(PLANT_STATE)
	if workMode == "seed" or workMode == "ripen" then
		transfer = true
	end
	
	local goodTemperature = true
	local liveTemperature = true
	local temperature = game.temperature
	if temperature <= config.plantDieTemp then
		liveTemperature = false	
		transfer = true		
	end
	if temperature < plantBuild.plantCfg.temperatureMin then
		goodTemperature = false	
	end
	
	local res = getBuildingResources(b)			
	for key,value in pairs(res) do
		local count = math.max(value + b:getPickReservation(key), 0)
		if transfer then
			if count > 0 then
				return false
			end
		end
	end
	
	if liveTemperature == false and plantBuild.plantCfg.harvestCount > 1 then
		return false
	end
	
	local camera = game.cameraManager.camera
	local viewDir = Vec3Normalize(camera.lookTo-camera.lookFrom)
	local height = 0.2
	local difNorm = Vec3Normalize(getPastureCellPos3D(cellNow.pos) + Vec3(0,height,0) - camera.lookFrom)
	if Vec3Dot(viewDir, difNorm) <= 0.8 then
		return false
	end
	
	local cellsCount = plantBuild.plantCellCount
	
	local widthCells = plantBuild.widthCells
	local lengthCells = plantBuild.lengthCells
	local widthPlant = plantBuild.horizontalBeds
	
	local nextStatus = true;
	while nextStatus do
		local workStatus = b:getWorkMode(PLANT_STATE)
		
		if workStatus == "seed" then
			
			local freeCellsCount = 0
			for i = 0, cellsCount-1, 1 do
				local cell = plantBuild:GetPlantCell(i)				
				if cell.state == SEED then				
					nextStatus = false					
					if cell.reserve == false then
						freeCellsCount = freeCellsCount + 1
					end
				end
			end
			
			if goodTemperature == false then -- or game.season == SEASON_FALL
				if freeCellsCount == cellsCount then
					nextStatus = false;
				else
					nextStatus = true;
				end
			else	
				if cellNow.state == SEED then	
					if checkAction == false then
						w:addTaskSetHint("@hint_plant_seed@")
			
						w:addTaskReservePlantCell(b, cellIndexNow, true)
					
						local cellPos = getPastureCellPos3D(cellNow.pos)
						if cellPos.x ~= w.pos.x or cellPos.z ~= w.pos.y then
							local dir = Vec2Normalize(Vec2(cellPos.x, cellPos.z) - w.pos)
							w:addTaskRotate(dir)
						end

						w:addTaskAttachModel("tools-shovel-01")
						w:addTaskPlayAnimationOnce("plant")
						w:addTaskPlayAnimation("stay")
						w:addTaskAttachModel("")
						w:addTaskCallEvent(b, "Seed", cellIndexNow, 0);
						w:addTaskReservePlantCell(b, cellIndexNow, false)	

						w:addTaskResetHint()
					end
					return true
				end
			end			
		end
		
		if workStatus == "growth" then
			if playerGrow(w, b, GROWTH, -20000, 150000, cellIndexNow, checkAction) then
				return true
			end
			
			for i = 0, cellsCount-1, 1 do
				local cell = plantBuild:GetPlantCell(i)
				if cell.state == GROWTH then			
					return false
				end
			end
		end
		
		if workStatus == "ripen" then
			if playerGrow(w, b, RIPEN, -20000, 110000, cellIndexNow, checkAction) then
				return true
			end
			
			for i = 0, cellsCount-1, 1 do
				local cell = plantBuild:GetPlantCell(i)
				if cell.state == RIPEN then			
					return false
				end
			end
		end
		
		if workStatus == "harvest" then	
			
			if cellNow.state == HARVEST then
				if cellNow.harvestCount == plantBuild.harvestCount then
					local plantCfg = cellNow.plantCfg
					if plantCfg ~= nil then
						if b.freeSpace >= cellNow.resourceCount then
							if checkAction == false then
								w:addTaskSetHint("@hint_plant_harvest@")
						
								w:addTaskReservePlantCell(b, cellIndexNow, true)
							
								local cellPos = getPastureCellPos3D(cellNow.pos)
								if cellPos.x ~= w.pos.x or cellPos.z ~= w.pos.y then
									local dir = Vec2Normalize(Vec2(cellPos.x, cellPos.z) - w.pos)
									w:addTaskRotate(dir)
								end
								if cellNow.plantCfg.field then
									if GRASS_PLANT[cell.plantCfg.name] then
									w:addTaskAttachModel("tools-scythe-01")
									w:addTaskPlayAnimationOnce("axecut")
									else
									w:addTaskAttachModel("tools-shovel-01")
									w:addTaskPlayAnimationOnce("plant")
									end
								else
									w:addTaskAttachModel("")
									w:addTaskWork(2000, "work")
								end				
								w:addTaskAttachModel("")
								w:addTaskCallEvent(b, "Harvest", cellIndexNow, 0)
								w:addTaskPlayAnimation("stay")
								w:addTaskAddResourceToBuilding(b, cellNow.plantCfg.outResource, cellNow.resourceCount)
								w:addTaskReservePlantCell(b, cellIndexNow, false)
								
								w:addTaskResetHint()
							end
							return true
						end
					end					
				end				
			end	
			
			for i = 0, cellsCount-1, 1 do
				local cell = plantBuild:GetPlantCell(i)				
				if cell.harvestCount == plantBuild.harvestCount then
					local plantCfg = cell.plantCfg
					if plantCfg ~= nil then
						if cell.state == HARVEST then
							nextStatus = false;
						end
					end					
				end
			end
					
		end
		
		if workStatus == "chop" then
		
			if cellNow.state == CHOP then
				if checkAction == false then
					w:addTaskSetHint("@hint_plant_chop@")
								
					w:addTaskReservePlantCell(b, cellIndexNow, true)
				
					local cellPos = getPastureCellPos3D(cellNow.pos)
					if cellPos.x ~= w.pos.x or cellPos.z ~= w.pos.y then
						local dir = Vec2Normalize(Vec2(cellPos.x, cellPos.z) - w.pos)
						w:addTaskRotate(dir)
					end
				
					if cellNow.plantCfg.field then
						if GRASS_PLANT[cell.plantCfg.name] then
						w:addTaskAttachModel("tools-scythe-01")
						w:addTaskPlayAnimationOnce("axecut")
						else
						w:addTaskAttachModel("tools-shovel-01")
						w:addTaskPlayAnimationOnce("hilling")
						end
					else
						w:addTaskAttachModel("tools-axe-01")
						w:addTaskWork(2000, "axecut")
					end
						
					w:addTaskCallEvent(b, "Chop", cellIndexNow, 0);
					w:addTaskReservePlantCell(b, cellIndexNow, false)	
					w:addTaskPlayAnimation("stay")	
					w:addTaskAttachModel("")
					
					w:addTaskResetHint()
				end
				return true
			end
			
			for i = 0, cellsCount-1, 1 do
				local cell = plantBuild:GetPlantCell(i)				
				if cell.state == CHOP then
					nextStatus = false;					
				end
			end
		end
		
		if nextStatus then	
			if workStatus == "seed" then
				b:setWorkMode(PLANT_STATE, "growth")
			elseif workStatus == "growth" then
				plantBuild.harvestCount = plantBuild.plantCfg.harvestCount
				b:setWorkMode(PLANT_STATE, "ripen")
			elseif workStatus == "ripen" then
				b:setWorkMode(PLANT_STATE, "harvest")
			elseif workStatus == "harvest" then
				plantBuild.harvestCount = plantBuild.harvestCount-1;
				if plantBuild.harvestCount > 0 then
					b:setWorkMode(PLANT_STATE, "ripen")
				else
					b:setWorkMode(PLANT_STATE, "chop")
				end
			elseif workStatus == "chop" then
				b:setWorkMode(PLANT_STATE, "seed")
				return false
			end
		end
	end	
end

Field = Class("Field", Plant)
Garden = Class("Garden", Plant)
